﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;

namespace InterLinq {
	/// <summary>
	/// Interface providing methods to get <see cref="IQueryable{T}"/>.
	/// </summary>
	public interface IQueryHandler {
		ReaderWriterLock LockOfDataSource { get; }

		/// <summary>
		/// Returns an <see cref="IQueryable{T}"/>.
		/// </summary>
		/// <typeparam name="T">Generic Argument of the returned <see cref="IQueryable{T}"/>.</typeparam>
		/// <returns>Returns an <see cref="IQueryable{T}"/>.</returns>
		IQueryable<T> GetTable<T>() where T : class;

		/// <summary>
		/// Returns an <see cref="IQueryable{T}"/>.
		/// </summary>
		/// <param name="type">Type of the returned <see cref="IQueryable{T}"/>.</param>
		/// <returns>Returns an <see cref="IQueryable{T}"/>.</returns>
		IQueryable GetTable(Type type);

		IEnumerable<Type> GetTableTypes();

		/// <summary>
		/// Tells the <see cref="IQueryHandler"/> to start a new the session.
		/// </summary>
		/// <returns>True, if the session creation was successful. False, if not.</returns>
		bool StartSession();

		/// <summary>
		/// Tells the <see cref="IQueryHandler"/> to close the current session.
		/// </summary>
		/// <returns>True, if the session closing was successful. False, if not.</returns>
		bool CloseSession();
	}
}