﻿using System;
using System.Collections.ObjectModel;
using System.Linq.Expressions;
using System.Runtime.Serialization;
using System.Text;
using InterLinq.Expressions.Helpers;
using InterLinq.Expressions.SerializableTypes;

namespace InterLinq.Expressions {
	/// <summary>
	/// A serializable version of <see cref="ListInitExpression"/>.
	/// </summary>
	[Serializable]
	[DataContract]
	public class SerializableListInitExpression : SerializableExpression {
		#region Constructors

		/// <summary>
		/// Default constructor for serialization.
		/// </summary>
		public SerializableListInitExpression() {
		}

		/// <summary>
		/// Constructor with an <see cref="ListInitExpression"/> and an <see cref="ExpressionConverter"/>.
		/// </summary>
		/// <param name="expression">The original, not serializable <see cref="Expression"/>.</param>
		/// <param name="expConverter">The <see cref="ExpressionConverter"/> to convert contained <see cref="Expression">Expressions</see>.</param>
		public SerializableListInitExpression(ListInitExpression expression, ExpressionConverter expConverter)
			: base(expression, expConverter) {
			m_newExpression = expression.NewExpression.MakeSerializable<SerializableNewExpression>(expConverter);
			m_initializers = expConverter.ConvertToSerializableObjectCollection<SerializableElementInit>(expression.Initializers);
		}

		#endregion

		#region Properties

		private ReadOnlyCollection<SerializableElementInit> m_initializers;
		private SerializableNewExpression m_newExpression;

		/// <summary>
		/// See <see cref="ListInitExpression.NewExpression"/>
		/// </summary>
		[DataMember]
		public SerializableNewExpression NewExpression {
			get { return m_newExpression; }
			set { m_newExpression = value; }
		}

		/// <summary>
		/// See <see cref="ListInitExpression.Initializers"/>
		/// </summary>
		[DataMember]
		public ReadOnlyCollection<SerializableElementInit> Initializers {
			get { return m_initializers; }
			set { m_initializers = value; }
		}

		#endregion

		#region ToString() Methods

		/// <summary>
		/// Builds a <see langword="string"/> representing the <see cref="Expression"/>.
		/// </summary>
		/// <param name="builder">A <see cref="System.Text.StringBuilder"/> to add the created <see langword="string"/>.</param>
		internal override void BuildString(StringBuilder builder) {
			if (builder == null) {
				throw new ArgumentNullException("builder");
			}
			m_newExpression.BuildString(builder);
			builder.Append(" {");
			int num = 0;
			int count = m_initializers.Count;
			while (num < count) {
				if (num > 0) {
					builder.Append(", ");
				}
				m_initializers[num].BuildString(builder);
				num++;
			}
			builder.Append("}");
		}

		#endregion
	}
}