﻿using System;
using System.Linq.Expressions;
using System.Runtime.Serialization;
using System.Text;
using InterLinq.Expressions.Helpers;
using InterLinq.Types;

namespace InterLinq.Expressions {
	/// <summary>
	/// A serializable version of <see cref="MemberExpression"/>.
	/// </summary>
	[Serializable]
	[DataContract]
	public class SerializableMemberExpression : SerializableExpression {
		#region Constructors

		/// <summary>
		/// Default constructor for serialization.
		/// </summary>
		public SerializableMemberExpression() {
		}

		/// <summary>
		/// Constructor with an <see cref="MemberExpression"/> and an <see cref="ExpressionConverter"/>.
		/// </summary>
		/// <param name="expression">The original, not serializable <see cref="Expression"/>.</param>
		/// <param name="expConverter">The <see cref="ExpressionConverter"/> to convert contained <see cref="Expression">Expressions</see>.</param>
		public SerializableMemberExpression(MemberExpression expression, ExpressionConverter expConverter)
			: base(expression, expConverter) {
			m_expression = expression.Expression.MakeSerializable(expConverter);
			m_member = InterLinqTypeSystem.Instance.GetInterLinqMemberInfo(expression.Member);
		}

		#endregion

		#region Properties

		private SerializableExpression m_expression;

		private InterLinqMemberInfo m_member;

		/// <summary>
		/// See <see cref="MemberExpression.Expression"/>
		/// </summary>
		[DataMember]
		public SerializableExpression Expression {
			get { return m_expression; }
			set { m_expression = value; }
		}

		/// <summary>
		/// See <see cref="MemberExpression.Member"/>
		/// </summary>
		[DataMember]
		public InterLinqMemberInfo Member {
			get { return m_member; }
			set { m_member = value; }
		}

		#endregion

		#region ToString() Methods

		/// <summary>
		/// Builds a <see langword="string"/> representing the <see cref="Expression"/>.
		/// </summary>
		/// <param name="builder">A <see cref="System.Text.StringBuilder"/> to add the created <see langword="string"/>.</param>
		internal override void BuildString(StringBuilder builder) {
			if (builder == null) {
				throw new ArgumentNullException("builder");
			}
			if (m_expression != null) {
				m_expression.BuildString(builder);
			}
			else {
				builder.Append(m_member.DeclaringType.Name);
			}
			builder.Append(".");
			builder.Append(m_member.Name);
		}

		#endregion
	}
}