﻿using System;
using System.Collections.ObjectModel;
using System.Linq.Expressions;
using System.Runtime.Serialization;
using System.Text;
using InterLinq.Expressions.Helpers;

namespace InterLinq.Expressions.SerializableTypes {
	/// <summary>
	/// A serializable version of <see cref="MemberListBinding"/>.
	/// </summary>
	[Serializable]
	[DataContract]
	public class SerializableMemberListBinding : SerializableMemberBinding {
		#region Constructors

		/// <summary>
		/// Default constructor for serialization.
		/// </summary>
		public SerializableMemberListBinding() {
		}

		/// <summary>
		/// Constructor with an <see cref="MemberListBinding"/> and an <see cref="ExpressionConverter"/>.
		/// </summary>
		/// <param name="memberListBinding">The original, not serializable <see cref="MemberBinding"/>.</param>
		/// <param name="expConverter">The <see cref="ExpressionConverter"/> to convert contained <see cref="Expression">Expressions</see>.</param>
		public SerializableMemberListBinding(MemberListBinding memberListBinding, ExpressionConverter expConverter)
			: base(memberListBinding, expConverter) {
			m_initializers =
				expConverter.ConvertToSerializableObjectCollection<SerializableElementInit>(memberListBinding.Initializers);
		}

		#endregion

		#region Properties

		private ReadOnlyCollection<SerializableElementInit> m_initializers;

		/// <summary>
		/// See <see cref="MemberListBinding.Initializers"/>
		/// </summary>
		[DataMember]
		public ReadOnlyCollection<SerializableElementInit> Initializers {
			get { return m_initializers; }
			set { m_initializers = value; }
		}

		#endregion

		#region ToString() Methods

		/// <summary>
		/// Builds a <see langword="string"/> representing the <see cref="MemberBinding"/>.
		/// </summary>
		/// <param name="builder">A <see cref="System.Text.StringBuilder"/> to add the created <see langword="string"/>.</param>
		internal override void BuildString(StringBuilder builder) {
			if (builder == null) {
				throw new ArgumentNullException("builder");
			}
			builder.Append(base.Member.Name);
			builder.Append(" = {");
			int num = 0;
			int count = m_initializers.Count;
			while (num < count) {
				if (num > 0) {
					builder.Append(", ");
				}
				m_initializers[num].BuildString(builder);
				num++;
			}
			builder.Append("}");
		}

		#endregion
	}
}