﻿#region Header

/*
Remote views based on linq
By

Pascal Hauser 
Dipl. Ing. in Informatik, Hochschule für Technik Rapperswil, 2006
Master Thesis, Hochschule für Technik Rapperswil, 2008-2010

and

Raphael Gfeller
Dipl. Ing. in Informatik, Hochschule für Technik Rapperswil, 2006
Master Thesis, Hochschule für Technik Rapperswil, 2008-2010

*/

#endregion

#region Usings

using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using InterLinq.Test.PD;
using Sebarf.Core.Interfaces;
using Sebarf.Diagnostics.Interfaces;
using Sebastof.Services.Interfaces;

#endregion

namespace InterLinq.Test.UI {
	/// <summary>
	/// is able to display and query for products based on IQueryable<IProduct> data
	/// </summary>
	public partial class ProductsView : UserControl {
		#region Public Methods

		/// <summary>
		/// ctr
		/// </summary>
		private ProductsView() {
			InitializeComponent();
		}

		/// <summary>
		/// ctr
		/// </summary>
		/// <param name="data">underlying data</param>
		/// <param name="ctx">used context</param>
		public ProductsView(IQueryable<IProduct> data, IServiceLocatorService ctx)
			: this() {
			Check.ArgumentNotNull(data, "data");
			m_data = data;
			DoASearch();
			m_ctx = ctx;
		}

		#endregion

		#region Public Fields

		/// <summary>
		/// defines how many entries are displayed
		/// </summary>
		public const int ViewSize = 25;

		#endregion

		#region Private Methods

		private object m_query = 1;

		private void Button_Click(object sender, RoutedEventArgs e) {
			DoASearch();
		}

		private void DoASearch() {
			m_searchValue = m_searchInput.Text;
			string searchValue = m_searchValue;
			m_pagedEntries = new AsnychPagedCollection<IProduct>(ViewSize, Dispatcher, GetElementCounts, GetElements);
			m_count.Content = m_pagedEntries.MaxElements;
			m_items.ItemsSource = m_pagedEntries.Elements;
			m_scrollBar.Maximum = m_pagedEntries.MaxElements;
			//m_scrollBar.Value = 0;
			m_scrollBar.SmallChange = 1;
			m_scrollBar.LargeChange = ViewSize;
			IProduct x = (from p in m_data.OrderBy(z => z.Name) where p.Name.Contains(searchValue) select p).First();
			//var y = (from p in m_data.Products.OrderBy(z=>z.Name)..Select((s, i) => new { S = s, Index = i }) where p.S.Name.Contains(searchValue) select p).First();
			if (x == null) {
				m_scrollBar.Value = 0;
			}
			else {
				int y = (from p in m_data where p.Name.CompareTo(x.Name) <= 0 select p).Count() - 1;
				m_scrollBar.Value = y;
			}
		}

		private int GetElementCounts() {
			return m_data.Count();
		}

		private Dictionary<int, object> GetElements(int from, int to) {
			var toReturn = new Dictionary<int, object>();
			string searchValue = m_searchValue;
			int index = from;
			foreach (IProduct entry in (from p in m_data orderby p.Name select p).Skip(from).Take(to - from)) {
				toReturn.Add(index, entry);
				index++;
			}
			return toReturn;
		}

		private void ScrollBar_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e) {
			m_pagedEntries.View((int)m_scrollBar.Value);
		}

		private void Button_Save(object sender, RoutedEventArgs e) {
			if (m_selectedProduct != null) {
				m_selectedProduct.Name = m_name.Text;
			}
			ReloadChangeSets();
		}

		private void Button_Delete(object sender, RoutedEventArgs e) {
			if (m_selectedProduct as InterLinqProduct != null) {
				m_ctx.GetRegisteredService<IChangeTrackingStorageService>().CurrentDepositedObjectsShouldDeleted.Add(
					m_selectedProduct);
			}
			ReloadChangeSets();
		}

		private void Button_New(object sender, RoutedEventArgs e) {
			if (m_ctx != null) {
				m_ctx.GetRegisteredService<IChangeTrackingStorageService>().CurrentDepositedObjectsHaveToBeAdded.Add(
					m_selectedProduct);
			}
			ReloadChangeSets();
		}

		private void m_items_MouseUp(object sender, MouseButtonEventArgs e) {
			m_selectedProduct = m_items.SelectedValue as IProduct;
			m_name.Text = string.Empty;
			if (m_selectedProduct != null) {
				m_name.Text = m_selectedProduct.Name;
			}
		}

		private void ReloadChangeSets() {
			if (m_ctx != null) {
				m_ChangeSetItems.ItemsSource =
					m_ctx.GetRegisteredService<IChangeHandlerDiagnosticsService>().GetAllPendingChanges();
			}
		}

		private void Button_Commit(object sender, RoutedEventArgs e) {
			if (m_ctx != null) {
				m_ctx.GetRegisteredService<IChangeHandlerService>().SubmitChanges();
			}
			ReloadChangeSets();
		}

		#endregion

		#region Private Fields

		private readonly IServiceLocatorService m_ctx;
		private readonly IQueryable<IProduct> m_data;
		private AsnychPagedCollection<IProduct> m_pagedEntries;
		private string m_searchValue;
		private IProduct m_selectedProduct;

		#endregion
	}
}