﻿#region Header

/*
Remote views based on linq
By

Pascal Hauser 
Dipl. Ing. in Informatik, Hochschule für Technik Rapperswil, 2006
Master Thesis, Hochschule für Technik Rapperswil, 2008-2010

and

Raphael Gfeller
Dipl. Ing. in Informatik, Hochschule für Technik Rapperswil, 2006
Master Thesis, Hochschule für Technik Rapperswil, 2008-2010

*/

#endregion

#region Usings

using System;
using System.Windows.Threading;
using Sebarf.Core.Internal;
using Sebarf.Diagnostics.Interfaces;
using Sebarf.Services.Interfaces;
using Sebarf.Session.Interfaces;

#endregion

namespace Sebarf.Session.Services {
	/// <summary>
	/// Description of the class
	/// </summary>
	public class SignOfLifeSenderService : Service {
		#region Public Properties

		[ServiceRequest]
		public IRequestRemoteHandlerEngineLocatorService RequestRemoteHandlerEngineLocatorService { get; set; }

		public TimeSpan SendSignOfLifeInterval { get; set; }

		#endregion

		#region Public Methods

		public SignOfLifeSenderService() {
			SendSignOfLifeInterval = TimeSpan.FromMinutes(1);
		}

		public override void DoSetup() {
			base.DoSetup();
			m_timer = new DispatcherTimer();
			m_timer.Interval = SendSignOfLifeInterval;
			m_timer.Tick += OnTimerTick;
			m_timer.Start();
		}

		public override void DoShutdown() {
			base.DoShutdown();
			m_timer.Tick -= OnTimerTick;
			m_timer.Stop();
		}

		#endregion

		#region Private Methods

		private void OnTimerTick(object sender, EventArgs e) {
			Logger.WriteInformation("Send sign of life");
			RequestRemoteHandlerEngineLocatorService.GetRemoteEngine().Process(new ResetSessionTimeoutRequest());
		}

		#endregion

		#region Private Fields

		private DispatcherTimer m_timer;

		#endregion
	}
}