﻿#region Header

/*
Remote views based on linq
By

Pascal Hauser 
Dipl. Ing. in Informatik, Hochschule für Technik Rapperswil, 2006
Master Thesis, Hochschule für Technik Rapperswil, 2008-2010

and

Raphael Gfeller
Dipl. Ing. in Informatik, Hochschule für Technik Rapperswil, 2006
Master Thesis, Hochschule für Technik Rapperswil, 2008-2010

*/

#endregion

#region Usings

using System;
using System.Collections.Generic;
using Sebarf.Diagnostics.Interfaces;
using Sebarf.Security.Interfaces;
using Sebarf.Services.Interfaces;
using Sebastof.Services.Interfaces;

#endregion

namespace Sebastof.Server.Services.Impl {
	/// <summary>
	/// Impl of <see cref="ISerializationDataHandlingStrategyService"/>
	/// Uses the <see cref="StandardDataHandlingStrategy"/>
	/// </summary>
	public class StandardSerializationDataHandlingStrategyService : Service, ISerializationDataHandlingStrategyService {
		#region Public Properties

		[ServiceRequest(IsOptional = true)]
		public IAuthorizationService AuthorizationService { get; set; }

		public bool EnableSecurity { get; set; }

		#endregion

		#region ISerializationDataHandlingStrategyService Members

		public Dictionary<string, object> GetDataToTransferFromObject(object obj) {
			var iObject = obj as ISebastofApplicationObject;
			Check.ArgumentNotNull(iObject, "iObject");
			var toReturn = new Dictionary<string, object>();
			foreach (var val in iObject.GetValues()) {
				if (!EnableSecurity || AuthorizationService.CheckAccess("ReadField", GetSecurityArguments(val.Key))) {
					toReturn.Add(val.Key, val.Value);
				}
				else {
					Logger.WriteDebug("Reading of " + val.Key + " has been denied");
				}
			}
			return toReturn;
		}

		public void RestoreObjectFromTransferedData(object obj, Dictionary<string, object> data) {
			throw new NotImplementedException();
		}

		#endregion

		#region Private Methods

		private Dictionary<string, object> GetSecurityArguments(string fieldName) {
			var toReturn = new Dictionary<string, object>();
			toReturn.Add("FieldName", fieldName);
			return toReturn;
		}

		#endregion
	}
}